package fr.nitorac.aurionweb.aurionweb;

import android.util.Log;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.nodes.Node;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import fr.nitorac.aurionweb.Utils;
import lombok.AccessLevel;
import lombok.Data;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;

@NoArgsConstructor
@Data
public class AurionSession {
    private static final String TAG = "AurionSession";

    @Getter(AccessLevel.NONE)
    @Setter(AccessLevel.NONE)
    private static final Pattern menuIdPattern = Pattern.compile("\\{PrimeFaces\\.ab\\(\\{s:\"(.*?)\"", Pattern.MULTILINE);

    @Getter(AccessLevel.NONE)
    @Setter(AccessLevel.NONE)
    private static final Pattern liSubmenuPattern = Pattern.compile("'form:sidebar_menuid':'(.*?)'", Pattern.MULTILINE);
    private static final List<String> elemsPopupLabel = Arrays.asList("cour", "interve", "ressourc");

    private String JSESSIONID = "";

    //Variables ID pour la connection et le menu (@see initConnection du AurionManager)
    @Setter(AccessLevel.PRIVATE)
    private String viewState = "";
    @Setter(AccessLevel.NONE)
    private String menuFormId = "";
    @Setter(AccessLevel.NONE)
    private String menuId = "";

    //Variables ID pour le ChoixPlanning
    @Setter(AccessLevel.NONE)
    private String submenuId = "";
    @Setter(AccessLevel.NONE)
    private String choixpFormId = "";

    //Variables ID pour le Planning
    @Setter(AccessLevel.NONE)
    private String choixpSubmitId = "";
    @Setter(AccessLevel.NONE)
    private String planningFormId = "";
    //Variables ID pour le popup d'un événement du Planning : Cours, Intervenants, Ressources, et Infos
    @Setter(AccessLevel.NONE)
    private ArrayList<String> elemsPopupId = new ArrayList<>(3);
    private Document lastDoc;

    private static String getViewState(Document doc) {
        Element view = doc.getElementById("j_id1:javax.faces.ViewState:0");
        if (view == null) {
            return null;
        }
        String test = view.attr("value");
        return test.isEmpty() ? /* CDATA Node */view.text() : test;
    }

    public void parseViewState(Document doc) {
        doc = setLastDoc(doc);
        String viewState = getViewState(doc);
        if (viewState == null) {
            Log.e(TAG, "Impossible d'analyser le ViewState pour : \n" + doc.outerHtml());
        }
        setViewState(viewState);
    }

    public boolean parseConnectMenuFormId(Document doc) {
        doc = setLastDoc(doc);
        //On va récupérer l'ID Aurionweb qui correspond à la form du menu
        String menuScript = doc.getElementsByAttributeValue("type", "text/javascript").stream().map(Node::outerHtml).filter(e -> e.contains("chargerSousMenu = function()")).findFirst().orElse("");
        Matcher menuMatcher = menuIdPattern.matcher(menuScript);
        menuFormId = menuMatcher.find() ? menuMatcher.group(1) : null;
        if (menuFormId == null) {
            Log.e(TAG, "Impossible de trouver l'ID menu dans : " + menuScript);
            return false;
        }
        return true;
    }

    public boolean parseConnectMenuId(Document doc) {
        doc = setLastDoc(doc);
        //Maintenant on va récupérer l'ID du menu qu'on souhaite, c'est à dire Emploi du temps
        // (Oui c'est bien un menu même si on cherche une balise avec submenu_... comme classe x) (c'est aurionweb)
        Element menuClass = doc.getElementsByAttributeValueContaining("class", "submenu_").stream().filter(e -> e.text().contains("Emploi du temps")).findFirst().orElse(null);
        menuId = menuClass != null ? menuClass.classNames().stream().filter(c -> c.startsWith("submenu_")).findFirst().orElse("") : null;
        if (menuId == null) {
            Log.e(TAG, "Impossible de trouver un menu avec 'Emploi du temps' comme texte !");
            return false;
        }
        return true;
    }

    public boolean parseConnectSubmenuId(Document doc) {
        doc = setLastDoc(doc);
        //On va maintenant trouver un ID de la forme '2_1' qui correspond à l'ID du lien "Planning d'un étudiant au choix"
        Element liSubmenuID = Jsoup.parse(doc.getElementById("form:sidebar").text()).getElementsByTag("li").stream().filter(e -> Utils.stripAccents(e.text()).equalsIgnoreCase("Planning d'un etudiant au choix")).findFirst().orElse(null);
        if (liSubmenuID != null) {
            Matcher submenuMatcher = liSubmenuPattern.matcher(liSubmenuID.child(0).attr("onclick"));
            if (submenuMatcher.find()) {
                submenuId = submenuMatcher.group(1);
                return true;
            }
        } else {
            Log.e(TAG, "Impossible de trouver un sous menu de tag 'li' avec 'Planning d'un étudiant au choix' comme texte !");
        }
        return false;
    }


    public boolean parseChoixPlanningFormId(Document doc) {
        doc = setLastDoc(doc);
        //On va trouver l'ID principal correspondant à la form de la page ChoixPlanning d'Aurionweb
        Element formDiv = doc.getElementsByAttributeValueContaining("class", "ui-datatable ui-widget").first();
        if (formDiv != null) {
            choixpFormId = formDiv.id();
            return true;
        }
        return false;
    }

    public boolean parseChoixPlanningSubmitBtnId(Document doc) {
        doc = setLastDoc(doc);
        //On va trouver l'ID du bouton 'Voir planning' qui est nécessaire dans les données d'une requête accédant à l'edt d'un user
        Element submitBtn = doc.getElementsByAttributeValue("type", "submit").stream().filter(e -> e.text().toLowerCase().contains("voir planning")).findFirst().orElse(null);
        if (submitBtn != null) {
            choixpSubmitId = submitBtn.id();
            return true;
        }
        return false;
    }

    public boolean parsePlanningFormId(Document doc) {
        doc = setLastDoc(doc);
        //On va trouver la form id principale de la page
        Element scheduleDiv = doc.getElementsByClass("schedule").first();
        if (scheduleDiv != null) {
            planningFormId = scheduleDiv.id();
            return true;
        }
        return false;
    }

    public void parseEventProps(Document doc) {
        elemsPopupId.clear();
        for (int i = 0; i < elemsPopupLabel.size(); i++) {
            elemsPopupId.add(null);
        }
        for (Element e : doc.getElementsByTag("a")) {
            if (e.children().size() == 0) {
                for (int i = 0; i < elemsPopupLabel.size(); i++) {
                    if (e.text().toLowerCase().contains(elemsPopupLabel.get(i))) {
                        //substring(1) pour retirer le # devant
                        elemsPopupId.set(i, doc.getElementById(e.attr("href").substring(1)).getElementsByAttributeValueContaining("id", "_data").first().id());
                    }
                }
            }
        }
    }


    public Document setLastDoc(Document doc) {
        return doc == null ? getLastDoc() : (lastDoc = doc);
    }
}
